// An example program how to use gif-h library.
//
// Creates one gif image in the current working directory. The gif contains an animation of a red square that moves
// one pixel to the right and down each 100ms.
//
// The pixels in the frames are in RGBA format with default color bit depth of 8 bits (uint8_t).
// [x,y] = (0, 0) is the top-left corner of the frame (image).
//
// The memory layout of a frame array:
// idx       x      y   color
// 0         0      0   red
// 1         0      0   green
// 2         0      0   blue
// 3         0      0   alpha
// 4         1      0   red
// 5         1      0   green
// 6         1      0   blue
// 7         1      0   alpha
// 8         2      0   red
// 9         2      0   green
// 10        2      0   blue
// 11        2      0   alpha
// ...
// 4*w-1     w-1    0   alpha
// 4*w       0      1   red

#include <iostream>
#include <string>

// Includes the library for creating gifs (the library is only this header file).
#include "gif.h"

using namespace std;

// Creates an empty frame array.
uint8_t *createFrame(int frameWidth, int frameHeight) {
    return new uint8_t[frameWidth * frameHeight * 4];
}

void setPixel(
        uint8_t *frame,
        int frameWidth,
        int x,
        int y,
        uint8_t red,
        uint8_t green,
        uint8_t blue,
        uint8_t alpha) {
    int pixelIndex = 4 * (y * frameWidth + x);

    // Color components.
    frame[pixelIndex + 0] = red;
    frame[pixelIndex + 1] = green;
    frame[pixelIndex + 2] = blue;
    frame[pixelIndex + 3] = alpha;
}

// Fills the whole frame with one color.
void fillFrame(
        uint8_t *frame, int frameWidth, int frameHeight,
        uint8_t red, uint8_t green, uint8_t blue, uint8_t alpha) {
    for (int y = 0; y < frameHeight; y++) {
        for (int x = 0; x < frameWidth; x++) {
            setPixel(frame, frameWidth, x, y, red, green, blue, alpha);
        }
    }
}

// Draws a filled rectangle into the frame. The rectangle is specified using its top-left corner and size.
void drawRectangle(
        uint8_t *frame, int frameWidth,
        int left, int top, int rectangleSize,
        uint8_t red, uint8_t green, uint8_t blue, uint8_t alpha) {
    for (int x = left; (x - left) < rectangleSize; x++) {
        for (int y = top; (y - top) < rectangleSize; y++) {
            setPixel(frame, frameWidth, x, y, red, green, blue, alpha);
        }
    }
}

int main() {
    // Where to store the gif image.
    string gifFile("output.gif");

    // Width (in pixels) of the frames in the gif image.
    int frameWidth = 200;

    // Height (in pixels) of the frames in the gif image.
    int frameHeight = 100;

    // Delay between frames. Gif-h expects the delay to be in hundredths of milliseconds, i.e., delay * 10 is in
    // milliseconds.
    int delay = 10;

    // Creates empty writer of gif image.
    GifWriter gifWriter;

    // Starts creating gif.
    GifBegin(&gifWriter, gifFile.c_str(), frameWidth, frameHeight, 0);

    // Creates a frame image. We will modify this frame to create an animation.
    auto frame = createFrame(frameWidth, frameHeight);

    // Creates the animation..
    for (int frameIndex = 0; frameIndex < 25; frameIndex++) {
        // Fills the frame with white color.
        fillFrame(frame, frameWidth, frameHeight, 255, 255, 255, 255);

        // Size of the rectangle and the position of its top-left corner.
        int rectangleSize = 50;
        int left = frameIndex;
        int top = frameIndex;

        // Draws a rectangle into the frame from top-left corner to the bottom-right corner.
        drawRectangle(frame, frameWidth, left, top, rectangleSize, 255, 0, 0, 255);

        // Writes the frame into gif. The content of the frame is copied, so after this function ends you can modify
        // the frame.
        GifWriteFrame(&gifWriter, frame, frameWidth, frameHeight, delay);
    }

    delete[] frame;

    // Finishes creating gif.
    GifEnd(&gifWriter);
}